package cn.gtmap.bdcdj.core.encrypt;

import cn.gtmap.bdcdj.core.encrypt.config.BaseConfig;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.tree.DefaultText;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DbEncryptXmlConfig {
    private Resource configUrl;
    private static final Logger LOGGER = LoggerFactory.getLogger(DbEncryptXmlConfig.class);
    public Resource getConfigUrl() {
        return configUrl;
    }
    @Autowired
    List<BaseConfig> baseConfig;
    //全局加解密字段
    private List<String> globalEnableFields ;
    //map中全局加解密字段的例外（在例外中不加解密）
    private List<Map> globalDisableEntityFields;
    //entity中全局加解密字段的例外（在例外中不加解密）
    private List<Map> globalDisableMapFields;
    //sql加解密字段
    private List<Map> sqlFileds;
    //sql仅加密字段
    private List<Map> sqlEncryptFileds;
    //sql仅解密字段
    private List<Map> sqlDecryptFileds;
    //实体加解密字段
    private List<Map> entityFileds;
    //sql参数为单独的string,对参数加密（等价于注解）
    private List<String> sqlSingleEncryptFileds;
    //sql返回值为单独的string,对返回值加密（等价于注解）
    private List<String> sqlSingleDecryptFileds;
    //接口加解密列表
    private List<String> interfaceEncryptList;
    private String interfaceSm2DecodeKey;//接口加解密时，sm2解密的私钥
    private String interfaceSm2EncodeKey;//接口加解密时，sm2加密的公钥
    private List<String> baseConfigOptions;

    public void setBaseConfig(List<BaseConfig> baseConfig) {
        this.baseConfig = baseConfig;
    }

    public void setGlobalEnableFields(List<String> globalEnableFields) {
        this.globalEnableFields = globalEnableFields;
    }

    public void setGlobalDisableEntityFields(List<Map> globalDisableEntityFields) {
        this.globalDisableEntityFields = globalDisableEntityFields;
    }

    public void setGlobalDisableMapFields(List<Map> globalDisableMapFields) {
        this.globalDisableMapFields = globalDisableMapFields;
    }

    public void setSqlFileds(List<Map> sqlFileds) {
        this.sqlFileds = sqlFileds;
    }

    public void setSqlEncryptFileds(List<Map> sqlEncryptFileds) {
        this.sqlEncryptFileds = sqlEncryptFileds;
    }

    public void setSqlDecryptFileds(List<Map> sqlDecryptFileds) {
        this.sqlDecryptFileds = sqlDecryptFileds;
    }

    public void setEntityFileds(List<Map> entityFileds) {
        this.entityFileds = entityFileds;
    }

    public void setSqlSingleEncryptFileds(List<String> sqlSingleEncryptFileds) {
        this.sqlSingleEncryptFileds = sqlSingleEncryptFileds;
    }

    public void setSqlSingleDecryptFileds(List<String> sqlSingleDecryptFileds) {
        this.sqlSingleDecryptFileds = sqlSingleDecryptFileds;
    }

    public void setInterfaceEncryptList(List<String> interfaceEncryptList) {
        this.interfaceEncryptList = interfaceEncryptList;
    }

    public void setInterfaceSm2DecodeKey(String interfaceSm2DecodeKey) {
        this.interfaceSm2DecodeKey = interfaceSm2DecodeKey;
    }

    public void setInterfaceSm2EncodeKey(String interfaceSm2EncodeKey) {
        this.interfaceSm2EncodeKey = interfaceSm2EncodeKey;
    }

    public void setBaseConfigOptions(List<String> baseConfigOptions) {
        this.baseConfigOptions = baseConfigOptions;
    }

    public List<String> getBaseConfigOptions() {
        if(CollectionUtils.isNotEmpty(this.baseConfigOptions)){
            return this.baseConfigOptions;
        }else{
            this.baseConfigOptions = getArrFields("BaseConfigOptions");
            return this.baseConfigOptions;
        }
    }

    public void cleanConfig(){
        setGlobalEnableFields(null);
        setGlobalDisableEntityFields(null);
        setGlobalDisableMapFields(null);
        setSqlFileds(null);
        setSqlEncryptFileds(null);
        setSqlDecryptFileds(null);
        setEntityFileds(null);
        setSqlSingleEncryptFileds(null);
        setSqlSingleDecryptFileds(null);
        setInterfaceEncryptList(null);
        setInterfaceSm2DecodeKey(null);
        setInterfaceSm2EncodeKey(null);
        setBaseConfigOptions(null);
    }
    public String getInterfaceSm2DecodeKey(){
        if(!StringUtils.isBlank(this.interfaceSm2DecodeKey)){
            return this.interfaceSm2DecodeKey;
        }else{
            this.interfaceSm2DecodeKey = getField("InterfaceSm2DecodeKey");
            return this.interfaceSm2DecodeKey;
        }
    }
    public String getInterfaceSm2EncodeKey(){
        if(!StringUtils.isBlank(this.interfaceSm2EncodeKey)){
            return this.interfaceSm2EncodeKey;
        }else {
            this.interfaceSm2EncodeKey = getField("InterfaceSm2EncodeKey");
            return this.interfaceSm2EncodeKey;
        }
    }
    public List<String> getInterfaceEncryptList(){
        if(CollectionUtils.isNotEmpty(this.interfaceEncryptList)){
            return this.interfaceEncryptList;
        }else {
            List<String> list = new ArrayList<String>();
            list = getFieldsInOneFiled("IneterfaceEcnrypt", "patternUrl");
            this.interfaceEncryptList = list;
            return list;
        }
    }
    public List<String> getSqlSingleEncryptFileds(){
        if(CollectionUtils.isNotEmpty(this.sqlSingleEncryptFileds)){
            return this.sqlSingleEncryptFileds;
        }else{
            this.sqlSingleEncryptFileds = getArrFields("SqlSingleEncryptFileds");
            return this.sqlSingleEncryptFileds;
        }
    }
    public List<String> getSqlSingleDecryptFileds(){
        if(CollectionUtils.isNotEmpty(this.sqlSingleDecryptFileds)){
            return this.sqlSingleDecryptFileds;
        }else{
            this.sqlSingleDecryptFileds = getArrFields("SqlSingleDecryptFileds");
            return this.sqlSingleDecryptFileds;
        }
    }

    public void setConfigUrl(Resource configUrl) {
        this.configUrl = configUrl;
    }


    public List<String> getGlobalEnableFields(){
        if(globalEnableFields == null) {
            globalEnableFields = getArrFields("GlobalEnableFields");
        }
        return globalEnableFields;
    }

    public List<Map> getGlobalDisableMapFields()  {
        if(globalDisableMapFields == null){
            List<String> globalDisableEntityFieldList = getArrFields("GlobalDisableMapFields");
            if(CollectionUtils.isNotEmpty(globalDisableEntityFieldList)){
                globalDisableMapFields = new ArrayList<Map>();
                for (String s : globalDisableEntityFieldList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityDisableMap = new HashMap<String,Object>();
                    entityDisableMap.put("mapid",mapid);
                    entityDisableMap.put("field",fieldName);
                    globalDisableMapFields.add(entityDisableMap);
                }
            }
        }
        return globalDisableMapFields;
    }

    public List<Map> getGlobalDisableEntityFields()  {
        if(globalDisableEntityFields == null){
            List<String> globalDisableEntityFieldList = getArrFields("GlobalDisableEntityFields");
            if(CollectionUtils.isNotEmpty(globalDisableEntityFieldList)){
                globalDisableEntityFields = new ArrayList<Map>();
                for (String s : globalDisableEntityFieldList) {
                    int i = s.lastIndexOf(".");
                    String beanName = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityDisableEntityMap = new HashMap<String,Object>();
                    entityDisableEntityMap.put("entity",beanName);
                    entityDisableEntityMap.put("field",fieldName);
                    globalDisableEntityFields.add(entityDisableEntityMap);
                }
            }
        }
        return globalDisableEntityFields;
    }

    public List<Map> getSqlFileds()  {
        if(sqlFileds == null){
            List<String> getSqlFiledsList = getArrFields("SqlFileds");
            if(CollectionUtils.isNotEmpty(getSqlFiledsList)){
                sqlFileds = new ArrayList<Map>();
                for (String s : getSqlFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlFiledsMap = new HashMap<String,Object>();
                    sqlFiledsMap.put("mapid",mapid);
                    sqlFiledsMap.put("field",fieldName);
                    sqlFileds.add(sqlFiledsMap);
                }
            }
        }
        return sqlFileds;
    }

    public List<Map> getSqlEncryptFileds()  {
        if(sqlEncryptFileds == null){
            List<String> getSqlEncryptFiledsList = getArrFields("SqlEncryptFileds");
            if(CollectionUtils.isNotEmpty(getSqlEncryptFiledsList)){
                sqlEncryptFileds = new ArrayList<Map>();
                for (String s : getSqlEncryptFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlEncryptFiledsMap = new HashMap<String,Object>();
                    sqlEncryptFiledsMap.put("mapid",mapid);
                    sqlEncryptFiledsMap.put("field",fieldName);
                    sqlEncryptFileds.add(sqlEncryptFiledsMap);
                }
            }
        }
        return sqlEncryptFileds;
    }

    public List<Map> getSqlDecryptFileds()  {
        if(sqlDecryptFileds == null){
            List<String> getSqlDecryptFiledsList = getArrFields("SqlDecryptFileds");
            if(CollectionUtils.isNotEmpty(getSqlDecryptFiledsList)){
                sqlDecryptFileds = new ArrayList<Map>();
                for (String s : getSqlDecryptFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map sqlDecryptFiledsMap = new HashMap<String,Object>();
                    sqlDecryptFiledsMap.put("mapid",mapid);
                    sqlDecryptFiledsMap.put("field",fieldName);
                    sqlDecryptFileds.add(sqlDecryptFiledsMap);
                }
            }
        }
        return sqlDecryptFileds;
    }

    public List<Map> getEntityFileds()  {
        if(entityFileds == null){
            List<String> getEntityFiledsList = getArrFields("EntityFileds");
            if(CollectionUtils.isNotEmpty(getEntityFiledsList)){
                entityFileds = new ArrayList<Map>();
                for (String s : getEntityFiledsList) {
                    int i = s.lastIndexOf(".");
                    String mapid = s.substring(0, i);
                    String fieldName = s.substring(i+1);
                    Map entityFiledsMap = new HashMap<String,Object>();
                    entityFiledsMap.put("entity",mapid);
                    entityFiledsMap.put("field",fieldName);
                    entityFileds.add(entityFiledsMap);
                }
            }
        }
        return entityFileds;
    }





    public List<String> getArrFields(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        List<String> result = new ArrayList<String>();
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    String value = globalEnableFields.elementTextTrim("value");
                    if(StringUtils.isNotBlank(value)){
                        String[] split = value.split(",");
                        if(split != null && split.length > 0){
                            for (String s : split) {
                                result.add(s);
                            }
                        }
                    }
                }
            }
        }
        return result;
    }
    public List<String> getFieldsInOneFiled(String fieldName,String secondFieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        List<String> result = new ArrayList<String>();
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    if(globalEnableFields.elements() != null && CollectionUtils.isNotEmpty(globalEnableFields.elements())){
                        List elements = globalEnableFields.elements();
                        for (Object obj : elements) {
                            Element element = (Element)obj;
                            List content = element.content();
                            if(CollectionUtils.isNotEmpty(content)){
                                DefaultText defaultText = (DefaultText)content.get(0);
                                if(defaultText != null && defaultText.getParent() != null
                                        && defaultText.getParent().getQName() != null
                                        && StringUtils.isNotBlank(defaultText.getParent().getQName().getName())
                                        && StringUtils.equals(secondFieldName,defaultText.getParent().getQName().getName()) ){
                                    String text = defaultText.getText();
                                    if(StringUtils.isNotBlank(text)){
                                        result.add(text);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return result;
    }
    public String getField(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        String value = "";
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    value = globalEnableFields.elementTextTrim("value");
                }
            }
        }
        return value;
    }
    public boolean getBooleanField(String fieldName){
        SAXReader sr = new SAXReader();
        Document document = null;
        try {
            document = sr.read(configUrl.getFile());
        } catch (Exception e) {
            LOGGER.error(null, e);
        }
        if(document != null){
            Element root = document.getRootElement();
            if(root != null) {
                Element globalEnableFields = root.element(fieldName);
                if(globalEnableFields != null){
                    //((DefaultText) ((DefaultElement) globalEnableFields).content).text.trim();
                    String stringVal = globalEnableFields.elementTextTrim("value");
                    if(StringUtils.isNotBlank(stringVal) && StringUtils.equals("true",stringVal)){
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * 从java文件中读取预置的加解密字段
     */
    public void addBaseConfigFileds() {
        List<String> baseConfigOptions = this.getBaseConfigOptions();
        if(CollectionUtils.isNotEmpty(baseConfig)){
            for (BaseConfig config : baseConfig) {
                String simpleName = config.getClass().getSimpleName().replace("Config", "").toLowerCase();
                //当配置中包含预置加密字段时，才读预置字段
                if(baseConfigOptions.contains(simpleName)){
                    //预置的全局加解密字段（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getGlobalEnableFields())){
                        if(CollectionUtils.isEmpty(globalEnableFields)){
                            globalEnableFields = new ArrayList<String>();
                        }
                        globalEnableFields.addAll(config.getGlobalEnableFields());
                    }
                    //预置的sql加解密字段（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getSqlFileds())){
                        if(CollectionUtils.isEmpty(sqlFileds)){
                            sqlFileds = new ArrayList<Map>();
                        }
                        sqlFileds.addAll(config.getSqlFileds());
                    }
                    //预置的sql加密字段（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getSqlEncryptFileds())){
                        if(CollectionUtils.isEmpty(sqlEncryptFileds)){
                            sqlEncryptFileds = new ArrayList<Map>();
                        }
                        sqlEncryptFileds.addAll(config.getSqlEncryptFileds());
                    }
                    //预置的sql解密字段（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getSqlDecryptFileds())){
                        if(CollectionUtils.isEmpty(sqlDecryptFileds)){
                            sqlDecryptFileds = new ArrayList<Map>();
                        }
                        sqlDecryptFileds.addAll(config.getSqlDecryptFileds());
                    }
                    //预置的实体加解密字段（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getEntityFileds())){
                        if(CollectionUtils.isEmpty(entityFileds)){
                            entityFileds = new ArrayList<Map>();
                        }
                        entityFileds.addAll(config.getEntityFileds());
                    }
                    //单一string作为参数加密(正向配置)
                    if(CollectionUtils.isNotEmpty(config.getSqlSingleEncryptFileds())){
                        if(CollectionUtils.isEmpty(sqlSingleEncryptFileds)){
                            sqlSingleEncryptFileds = new ArrayList<String>();
                        }
                        sqlSingleEncryptFileds.addAll(config.getSqlSingleEncryptFileds());
                    }
                    //单一string作为返回值解密（正向配置）
                    if(CollectionUtils.isNotEmpty(config.getSqlSingleDecryptFileds())){
                        if(CollectionUtils.isEmpty(sqlSingleDecryptFileds)){
                            sqlSingleDecryptFileds = new ArrayList<String>();
                        }
                        sqlSingleDecryptFileds.addAll(config.getSqlSingleDecryptFileds());
                    }
                    //全局加密字段在sql中的反向配置（配置在其中的不进行加解密处理） 反向配置
                    //不需要做特殊处理逻辑，因为在判断是否为加解密字段时，判断顺序为：是否为全局加解密字段-》是否为全局非加解密字段-》是否为加解密字段，所以是否为加解密字段本身优先级高于全局配置，不需要在此处根据已有的是否为加解密字段配置调整是否为全局非加解密字段
                    if(CollectionUtils.isNotEmpty(config.getGlobalDisableMapFields())){
                        if(CollectionUtils.isEmpty(globalDisableMapFields)){
                            globalDisableMapFields = new ArrayList<Map>();
                        }
                        globalDisableMapFields.addAll(config.getGlobalDisableMapFields());
                    }
                    //全局姐姐们字段在entity中的反向配置（配置在其中的不进行加解密处理） 反向配置
                    //不做特殊逻辑处理，原因同上
                    if(CollectionUtils.isNotEmpty(config.getGlobalDisableEntityFields())){
                        if(CollectionUtils.isEmpty(globalDisableEntityFields)){
                            globalDisableEntityFields = new ArrayList<Map>();
                        }
                        globalDisableEntityFields.addAll(config.getGlobalDisableEntityFields());
                    }
                }
            }
        }
    }
}
